(function (PLUGIN_ID) {
    'use strict';


    // xxxxxTxxxxZ 文字列を日本時間のISO8601文字列に変換
    function convDate(day_str) {

        var date = new Date(day_str);

        var year = date.getFullYear();
        var month = ('0' + (date.getMonth() + 1)).slice(-2); // 0埋めした2桁の月
        var day = ('0' + date.getDate()).slice(-2); // 0埋めした2桁の日
        var hours = ('0' + date.getHours()).slice(-2); // 0埋めした2桁の時
        var minutes = ('0' + date.getMinutes()).slice(-2); // 0埋めした2桁の分
        var seconds = ('0' + date.getSeconds()).slice(-2); // 0埋めした2桁の秒

        var formattedDate = year + '-' + month + '-' + day + ' ' + hours + ':' + minutes + ':' + seconds;

        return formattedDate;
    }


    // sqlite3のデータクラス
    class sql_db_class {

        // コンストラクタ
        constructor() {
            this.SQL;
            this.db;
            this.headers = [];
        }


        // 初期化
        async init() {
            this.SQL = await initSqlJs();
            this.db = new this.SQL.Database();
        }


        // テーブルを作成
        async createTable(recs) {

            const objRecord = recs[0];

            let keys = Object.keys(objRecord);
            this.headers = keys.map((key) => {
                return key.replace('$', '');
            });


            let sql_create = 'create table tbl2 (\n';
            Object.entries(objRecord).forEach(([key, value]) => {
                if (value['type'] === "NUMBER" || value['type'] === "__ID__" || value['type'] === 'RECORD_NUMBER') {
                    sql_create += `${key} number,\n`.replace('$', '');
                }
                else if (value['type'] === "DATE") {
                    sql_create += `${key} timestamp,\n`.replace('$', '');
                }
                else {
                    sql_create += `${key} text,\n`.replace('$', '');
                }
            });
            sql_create = sql_create.slice(0, -2) + ');';
            // console.log(sql_create);
            this.db.run(sql_create);

            await this.getTableInfo();
            await this.insert_sql(recs);
        }


        // テーブル情報を取得
        async getTableInfo() {
            const tableInfo = await this.db.exec("PRAGMA table_info('tbl2');");
            // console.log(tableInfo);
        }


        // データを挿入
        async insert_sql(recs) {

            function getValue([key, value]) {
                if (value['type'] === "MODIFIER" || value['type'] === "CREATOR") {
                    return {
                        [key]: value['value']['name']
                    };
                }
                else if (value['type'] === "DATETIME" || value['type'] === "CREATED_TIME" || value['type'] === "UPDATED_TIME") {
                    return {
                        [key]: convDate(value['value'])
                    };
                }
                else if (value['type'] === "FILE") {

                    let file_str = '';
                    value['value'].forEach(file => {
                        file_str += file.name + ',\n';
                    });

                    return {
                        [key]: file_str
                    };
                }
                else {
                    return {
                        [key]: value['value']
                    };
                }
            }

            recs.forEach(rec => {
                const rec2 = Object.entries(rec).map(getValue);

                let fields = '';
                let values = '';
                rec2.forEach(item => {
                    getRecords
                    const key = Object.keys(item)[0];
                    const value = Object.values(item)[0];
                    fields += `${key}, `.replace('$', '');
                    values += `'${value}', `;
                });
                const sql_insert = `insert into tbl2 (${fields.slice(0, -2)}) values (${values.slice(0, -2)});`;
                this.db.run(sql_insert);
            });
        }


        // seletしてhtmlテーブルを返す
        async select_sql(select_sql) {

            const allData = await this.db.exec(select_sql);
            this.headers = allData[0].columns;
            // console.log(head);

            const tableElement = document.createElement('table');
            tableElement.className = 'table_a';
            const tableHead = document.createElement('thead');
            const headerRow = document.createElement('tr');

            this.headers.forEach(headerText => {
                const headerCell = document.createElement('th');
                headerCell.className = 'th_a';
                headerCell.textContent = headerText;
                headerCell.style.border = '1px solid black';
                headerRow.appendChild(headerCell);
            });

            tableHead.appendChild(headerRow);
            tableElement.appendChild(tableHead);

            const tableBody = document.createElement('tbody');
            allData[0].values.forEach(rowData => {
                const row = document.createElement('tr');
                rowData.forEach(cellData => {
                    const cell = document.createElement('td');
                    cell.className = 'td_a';
                    cell.textContent = cellData;
                    cell.style.border = '1px solid black';
                    row.appendChild(cell);
                });
                tableBody.appendChild(row);
            });

            tableElement.appendChild(tableBody);
            tableElement.style.border = '1px solid black';

            return tableElement;
        }


        async select_sql_data(select_sql) {

            const allData = await this.db.exec(select_sql);
            return allData;

        }


    }


    // レコードを取得する
    async function getRecords(app_id, opt_offset, opt_limit, opt_records) {

        const offset = opt_offset || 0;
        const limit = opt_limit || RECORD_LIMIT;
        let allRecords = opt_records || [];

        const params = {
            app: app_id,
            query: `limit ${limit} offset ${offset}`,
        };

        try {
            const resp = await kintone.api("/k/v1/records", "GET", params);

            allRecords = allRecords.concat(resp.records);
            if (resp.records.length === limit) {
                return getRecords(app_id, offset + limit, limit, allRecords);
            }

            return allRecords;
        } catch (error) {
            console.error(error);
            throw new Error("Error fetching records");
        }
    }


    // 全レコードを取得する(wrap関数)
    async function getAllRecords(app_id) {
        return await getRecords(app_id, 0, 500);
    }


    // htmlを置換する
    function replaceAll(html, objReplace) {

        for (let key in objReplace) {
            html = html.replace(new RegExp(key, "g"), objReplace[key]);
        }

        return html;
    }


    kintone.events.on('app.record.index.show', async function (event) {

        const sql_db = new sql_db_class();
        await sql_db.init();
        const records = await getAllRecords(kintone.app.getId());
        // console.log(records);
        await sql_db.createTable(records);


        var config = kintone.plugin.app.getConfig(PLUGIN_ID);
        var spaceElement = kintone.app.getHeaderSpaceElement();

        const html = config['text_html'];
        const objHtml = JSON.parse(html);
        console.log(objHtml);

        if (event.viewName != "SQL" && event.viewName != "テンプレート") {


            for (var index_name in objHtml) {
                if (event.viewName == index_name) {
                    const template_html = objHtml[index_name]['template'];
                    console.log(index_name);
                    console.log(template_html);
                    // const html2 = replaceAll(objHtml[index_name], objReplace);
                    // template.innerHTML = html2;
                    // template.style.padding = '5px';
                    let objReplace = objHtml[index_name]['sql'];
                    // for (var keyword in sqls) {
                    //     console.log(keyword);
                    //     console.log(sqls[keyword]);
                    // }
                    for (let key in objReplace) {
                        const select_data = await sql_db.select_sql_data(objReplace[key]);
                        const val = select_data[0].values[0][0];
                        objReplace[key] = val;
                    }

                    const html2 = replaceAll(template_html, objReplace);
                    const template = document.getElementById('view');
                    template.innerHTML = html2;
                    template.style.padding = '5px';


                }
                // const template = document.getElementById(key);
            }

            // const text_sql = config['text_sql'];
            // console.log(html);
            // console.log(text_sql);
            // const objReplace = JSON.parse(text_sql);

            // console.log(objReplace);

            // let objReplace = {
            //     "{予定1}": "select count(*) cnt from tbl2",
            //     "{予定2}": "select count(*) cnt from tbl2 where 重量 > 60",
            //     "{予定3}": "select count(*) cnt from tbl2 where 納期 >= '2024-01-15'",
            // };

            // for (let key in objReplace) {
            //     const select_data = await sql_db.select_sql_data(objReplace[key]);
            //     const val = select_data[0].values[0][0];
            //     objReplace[key] = val;
            // }

            // // const select_sql = 'select count(*) cnt from tbl2';
            // // // const select_sql = 'select * from tbl2';
            // // const select_data = await sql_db.select_sql_data(select_sql);
            // // const val=select_data[0].values[0][0];

            // // console.log(`val=${val}`);


            // // const objReplace = {
            // //     "{予定1}": val,
            // //     "{予定2}": "2024/2/3",
            // //     "{予定3}": "2024/4/12",
            // // };


            // const template = document.getElementById('view');

            // const html2 = replaceAll(html, objReplace);
            // // console.log(html2);
            // template.innerHTML = html2;
            // template.style.padding = '5px';

            return;
        }
        else if (event.viewName == "SQL") {

            const viewElement = document.getElementById('view');
            viewElement.style.padding = '5px';

            const inputElement = document.createElement('input');
            inputElement.type = 'text';
            inputElement.id = 'sql';
            inputElement.className = 'kintoneplugin-input-text';
            inputElement.style.margin = '5px';
            inputElement.setAttribute('size', '70');
            inputElement.setAttribute('value', 'select * from tbl2');
            viewElement.appendChild(inputElement);

            const buttonElement = document.createElement('button');
            buttonElement.textContent = '実行';
            buttonElement.id = 'btn2';
            buttonElement.className = 'kintoneplugin-button-dialog-ok';
            buttonElement.style.margin = '5px';
            viewElement.appendChild(buttonElement);
            const tableSpaceElement = document.createElement('div');
            tableSpaceElement.id = 'table_space';
            viewElement.appendChild(tableSpaceElement);




            document.getElementById('btn2').addEventListener('click', async () => {
                // console.log('Button clicked');

                // const records = await getAllRecords(kintone.app.getId());
                // console.log(records);


                // await sql_db.createTable(records);
                const select_sql = document.getElementById('sql').value;
                // const select_sql = 'select * from tbl2';
                try {
                    const table_element = await sql_db.select_sql(select_sql);
                    const tableArea = document.getElementById('table_space');

                    tableArea.innerHTML = table_element.outerHTML;
                }
                catch (error) {
                    // console.log(error);

                    window.alert(error.message);
                }
                // const table_element = await sql_db.select_sql(select_sql);
                // const tableArea = document.getElementById('table_space');

                // tableArea.innerHTML = table_element.outerHTML;
            });

        }




    });

})(kintone.$PLUGIN_ID);
